include_guard(GLOBAL)

if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/sdk)
	file(MAKE_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}/sdk)
endif()

if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/inc)
	file(MAKE_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}/inc)
endif()

message("--    SDK_TYPE : $ENV{SDK_TYPE}")
message("-- SDK_INCLUDE : $ENV{SDK_INCLUDE}")
message("--     SDK_LIB : $ENV{SDK_LIB}")

# option(ENABLE_HAL_DRIVER "Build & Link with HAL drivers" OFF)
option(ENABLE_STD_DRIVER "Build & Link with stdandrd peripheral drivers" ON)
option(ENABLE_SEGGER_RTT "Build & Link with SEGGER RTT support, J-Link required!" OFF)
set(TARGET_CPU_FLOAT_ABI soft CACHE STRING "Select float ABI: none, soft, softfp, hard")
set(TARGET_SPECS nano CACHE STRING "Select specs: none, nosys, nano, rdimon, rdimon-v2m")
set(TARGET_HSE_VALUE auto CACHE STRING "Select HSE Speed: auto, <number>")
set(TARGET_HSI_VALUE auto CACHE STRING "Select HSI Speed: auto, <number>")

if ("${TARGET_CPU}" STREQUAL "")
	message(FATAL_ERROR "TARGET_CPU was not specified!")
endif()

string(TOLOWER "${TARGET_CPU}" TARGET_CPU)

# set(GD32_FLASH_ORIGIN 0x8000000)
# set(GD32_FLASH_LENGTH 0x7FFFF)
# set(GD32_RAM_ORIGIN 0x2000000)
# set(GD32_RAM_LENGTH 0x7FFFF)

if ("${TARGET_CPU}" MATCHES "^(n32g030|n32g031|n32g032).([0-9a-z])$")
	set(_N32_CPU_PREFIX "${CMAKE_MATCH_1}")
	set(TARGET_CPU_TYPE_COMMON "${_N32_CPU_PREFIX}")
	set(TARGET_CPU_TYPE_DEF "${_N32_CPU_PREFIX}")
	set(TARGET_CPU_SYSTEM_C "${_N32_CPU_PREFIX}")
	set(TARGET_CPU_STARTUP_ASM "${_N32_CPU_PREFIX}")
	set(TARGET_CPU_LD_SCRIPT "${_N32_CPU_PREFIX}")
	set(TARGET_CPU_TYPE cortex-m0)
elseif ("${TARGET_CPU}" MATCHES "^(n32g430|n32g401).([0-9a-z])$")
	set(_N32_CPU_PREFIX "${CMAKE_MATCH_1}")
	set(TARGET_CPU_TYPE_COMMON "${_N32_CPU_PREFIX}")
	set(TARGET_CPU_TYPE_DEF "${_N32_CPU_PREFIX}")
	set(TARGET_CPU_SYSTEM_C "${_N32_CPU_PREFIX}")
	set(TARGET_CPU_STARTUP_ASM "${_N32_CPU_PREFIX}")
	set(TARGET_CPU_LD_SCRIPT "${_N32_CPU_PREFIX}")
	set(TARGET_CPU_TYPE cortex-m4)
elseif ("${TARGET_CPU}" MATCHES "^(n32[gl]4[035]).([0-9a-z])$")
	set(_N32_CPU_PREFIX "${CMAKE_MATCH_1}")
	set(TARGET_CPU_TYPE_COMMON "${_N32_CPU_PREFIX}x")
	set(TARGET_CPU_TYPE_DEF "${_N32_CPU_PREFIX}x")
	set(TARGET_CPU_SYSTEM_C "${_N32_CPU_PREFIX}x")
	set(TARGET_CPU_STARTUP_ASM "${_N32_CPU_PREFIX}x")
	set(TARGET_CPU_LD_SCRIPT "${_N32_CPU_PREFIX}x")
	set(TARGET_CPU_TYPE cortex-m4)
else()
	message(FATAL_ERROR "Unsupported device [${TARGET_CPU}]!")
endif()

message("--      Device : " ${TARGET_CPU})
message("--         CPU : " ${TARGET_CPU_TYPE})


if(CLANG)
	add_compile_options(--target=arm-none-eabi -Wno-unused-command-line-argument)
	add_link_options(--target=arm-none-eabi -Wno-unused-command-line-argument)
endif()

add_compile_definitions(${TARGET_CPU_TYPE_COMMON} ${TARGET_CPU_TYPE_DEF})
add_compile_options(
	-mthumb
	-mcpu=${TARGET_CPU_TYPE}
	-Wfatal-errors
	-fno-exceptions
	-fno-unwind-tables
	-fno-stack-check
)

set(CMAKE_CXX_FLAGS	"${CMAKE_CXX_FLAGS} -fno-rtti -fno-exceptions -fuse-cxa-atexit")
add_link_options(
	# -DflashOrigin=${GD32_FLASH_ORIGIN}
	# -DflashLength=${GD32_FLASH_LENGTH}
	# -DramOrigin=${GD32_RAM_ORIGIN}
	# -DramLength=${GD32_RAM_LENGTH}
	-mthumb
	-mcpu=${TARGET_CPU_TYPE}
	-Wl,--print-memory-usage
	-Wl,--warn-once
	${EMBEDED_GCC_EXTRA_LD_FLAGS}
	-L"$ENV{SDK_LIB}"
)

message("--       Specs : " ${TARGET_SPECS})
select_embeded_spec(${TARGET_SPECS})

include_directories(${CMAKE_CURRENT_SOURCE_DIR}/inc $ENV{SDK_INCLUDE})

if ("${TARGET_CPU_FLOAT_ABI}" MATCHES "soft|softfp|hard")
	add_compile_options(-mfloat-abi=${TARGET_CPU_FLOAT_ABI})
	add_link_options(-mfloat-abi=${TARGET_CPU_FLOAT_ABI})
	message("--   Float ABI : " ${TARGET_CPU_FLOAT_ABI})
elseif(NOT "${TARGET_CPU_FLOAT_ABI}" STREQUAL "none")
	message("${_WARNING_PREFIX} : Invalid Float ABI ${TARGET_CPU_FLOAT_ABI}")
endif()

if ("${TARGET_HSE_VALUE}" STREQUAL "auto")
	message("--  HSE_VALUE : auto")
elseif("${TARGET_HSE_VALUE}" MATCHES "^[0-9]+$")
	add_compile_definitions(HSE_VALUE=${TARGET_HSE_VALUE})
	message("--  HSE_VALUE : " ${TARGET_HSE_VALUE})
else()
	message("${_WARNING_PREFIX} : Invalid HSE_VALUE ${TARGET_HSE_VALUE}")
endif()

if ("${TARGET_HSI_VALUE}" STREQUAL "auto")
	message("--  HSI_VALUE : auto")
elseif("${TARGET_HSI_VALUE}" MATCHES "^[0-9]+$")
	add_compile_definitions(HSI_VALUE=${TARGET_HSI_VALUE})
	message("--  HSI_VALUE : " ${TARGET_HSI_VALUE})
else()
	message("${_WARNING_PREFIX} : Invalid HSI_VALUE ${TARGET_HSI_VALUE}")
endif()

set(N32_SDK)

if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/sdk/${TARGET_CPU_LD_SCRIPT}_FLASH.ld)
	if (EXISTS $ENV{SDK_LDSCRIPT_ROOT}/${TARGET_CPU_LD_SCRIPT}_FLASH.ld)
		file(COPY_FILE
			$ENV{SDK_LDSCRIPT_ROOT}/${TARGET_CPU_LD_SCRIPT}_FLASH.ld
			${CMAKE_CURRENT_SOURCE_DIR}/sdk/${TARGET_CPU_LD_SCRIPT}_FLASH.ld
		)
	endif()
endif()

# copy xxx_libopt.h
if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/inc/${TARGET_CPU_SYSTEM_C}_conf.h)
	file(COPY_FILE
		$ENV{SDK_TEMPLATE_ROOT}/${TARGET_CPU_SYSTEM_C}_conf.h
		${CMAKE_CURRENT_SOURCE_DIR}/inc/${TARGET_CPU_SYSTEM_C}_conf.h
	)
endif()

# copy system_xxxx.c
if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/sdk/system_${TARGET_CPU_SYSTEM_C}.c)
	file(COPY_FILE
		$ENV{SDK_CMSIS_DEVICES_SRC}/system_${TARGET_CPU_SYSTEM_C}.c
		${CMAKE_CURRENT_SOURCE_DIR}/sdk/system_${TARGET_CPU_SYSTEM_C}.c
	)
endif()

# copy startup_xxxx.s
if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/sdk/startup_${TARGET_CPU_STARTUP_ASM}.s)
	file(COPY_FILE
		$ENV{SDK_CMSIS_DEVICES_SRC}/startup/startup_${TARGET_CPU_STARTUP_ASM}_gcc.s
		${CMAKE_CURRENT_SOURCE_DIR}/sdk/startup_${TARGET_CPU_STARTUP_ASM}.s
	)
endif()

# find ld script
if (EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/SDK/${TARGET_CPU_LD_SCRIPT}_FLASH.ld)
	add_link_options(-Wl,-T,${CMAKE_CURRENT_SOURCE_DIR}/SDK/${TARGET_CPU_LD_SCRIPT}_FLASH.ld)
	message("--   LD Script : sdk/${TARGET_CPU_LD_SCRIPT}_FLASH.ld")
elseif(EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/SDK/${TARGET_CPU_LD_SCRIPT}_FLASH.ld)
	add_link_options(-Wl,-T,${CMAKE_CURRENT_SOURCE_DIR}/SDK/${TARGET_CPU_LD_SCRIPT}_FLASH.ld)
	message("--   LD Script : sdk/${TARGET_CPU_LD_SCRIPT}_FLASH.ld")
else()
	set(_LINK_FILE_PREFIX ${TARGET_CPU})
	string(TOUPPER "${_LINK_FILE_PREFIX}" _LINK_FILE_PREFIX)
	string(REGEX REPLACE "^N32(....).*" "N32\\1xx" _LINK_FILE_PREFIX "${_LINK_FILE_PREFIX}")
	if(EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/SDK/${_LINK_FILE_PREFIX}_FLASH.ld)
		add_link_options(-Wl,-T,${CMAKE_CURRENT_SOURCE_DIR}/SDK/${_LINK_FILE_PREFIX}_FLASH.ld)
		message("--   LD Script : sdk/${_LINK_FILE_PREFIX}_FLASH.ld")
	else()
		message("${_WARNING_PREFIX} : LD script not found!")
	endif()
endif()

if (ENABLE_HAL_DRIVER)
	message("--  HAL Driver : enabled")
	file(GLOB_RECURSE HAL_DRIVER_SOURCES
		LIST_DIRECTORIES false
		$ENV{SDK_HAL_DRIVER_SRC}/*_hal_*.c
		$ENV{SDK_HAL_DRIVER_SRC}/*_hal.c
	)
	list(TRANSFORM HAL_DRIVER_SOURCES REPLACE "\\\\" "/")

	# Prepare HAL templates
	set(HAL_DRIVER_SOURCES_TEMPLATES ${HAL_DRIVER_SOURCES})
	list(FILTER HAL_DRIVER_SOURCES_TEMPLATES INCLUDE REGEX .*_template\.c)
	list(FILTER HAL_DRIVER_SOURCES EXCLUDE REGEX .*_template\.c)

	if (HAL_DRIVER_SOURCES_TEMPLATES)
		if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/hal)
			file(MAKE_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}/hal)
			foreach(TEMPLATE_ITEM IN LISTS HAL_DRIVER_SOURCES_TEMPLATES)
				string(REGEX REPLACE ".*/(.+)_template\\.c" "\\1.c" LOCAL_FILE_NAME "${TEMPLATE_ITEM}")
				file(COPY_FILE
					${TEMPLATE_ITEM}
					${CMAKE_CURRENT_SOURCE_DIR}/hal/${LOCAL_FILE_NAME}-template
				)
			endforeach()
		endif()
	endif()

	add_library(hal-driver STATIC
		${HAL_DRIVER_SOURCES}
	)

	list(APPEND N32_SDK hal-driver)

	string(TOLOWER "${TARGET_CPU_TYPE_COMMON}" _HAL_CONF_FILE_PREFIX)
	if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/inc/${_HAL_CONF_FILE_PREFIX}_hal_conf.h)
		if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/inc)
			file(MAKE_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}/inc)
		endif()
		file(COPY_FILE
			$ENV{SDK_HAL_DRIVER_SRC}/../Inc/${_HAL_CONF_FILE_PREFIX}_hal_conf_template.h
			${CMAKE_CURRENT_SOURCE_DIR}/inc/${_HAL_CONF_FILE_PREFIX}_hal_conf.h
		)
	endif()
else()
	# message("--  HAL Driver : disabled")
endif()

if (ENABLE_STD_DRIVER)
	message("--  STD Driver : enabled")
	file(GLOB_RECURSE STD_DRIVER_SOURCES
		LIST_DIRECTORIES false
		$ENV{SDK_DRIVER_SRC}/*.c
	)
	list(TRANSFORM STD_DRIVER_SOURCES REPLACE "\\\\" "/")
	add_library(std-driver STATIC
		${STD_DRIVER_SOURCES}
	)
	list(APPEND N32_SDK std-driver)
else()
	message("--  STD Driver : disabled")
endif()

include(${CMAKE_CURRENT_LIST_DIR}/segger-rtt.cmake)

file(GLOB_RECURSE SOURCE_FILES
	LIST_DIRECTORIES false
	${SEGGER_RTT_SOURCES}
	sdk/*.cxx
	sdk/*.cpp
	sdk/*.cc
	sdk/*.c
	sdk/*.s
	src/*.cxx
	src/*.cpp
	src/*.cc
	src/*.c
	src/*.s
	hal/*.cxx
	hal/*.cpp
	hal/*.cc
	hal/*.c
	hal/*.s
)
